@@ -124,8 +124,10 @@ class TargetCanvas(DisplayTarget):
                 x1, y1, x2, y2, fill = args
                 x1, y1 = self.__transform_coords(x1, y1)
                 x2, y2 = self.__transform_coords(x2, y2)
+                w = x2 - x1
+                h = y2 - y1
                 self.__buffer.draw_rectangle(self.__buffer_gc, int(fill),
-                                             x1, y1, x2, y2)
+                                             x1, y1, w, h)
 
                 
         #end for
