@@ -1,6 +1,7 @@
 from ContainerTarget import ContainerTarget
 from utils.Hash2D import Hash2D
 from utils.datatypes import *
+from utils import vfs
 
 import gtk
 import os
@@ -133,10 +134,24 @@ class TargetGroup(ContainerTarget):
     #
     def __set_background(self, uri):
 
+        if (not uri):
+            self.__set_color("#00000000")
+            return
+        
         uri = os.path.join(self.get_path(), uri)
-        self.__pixbuf = gtk.gdk.pixbuf_new_from_file(uri)
-        nil, nil, w, h = self.get_geometry()
-        self.__tile_background(w, h)
+        if (vfs.exists(uri)):
+            fd = vfs.open(uri, "r")
+            data = vfs.read_all(fd)
+            fd.close()
+            loader = gtk.gdk.PixbufLoader()
+            loader.write(data, len(data))
+            loader.close()
+            self.__pixbuf = loader.get_pixbuf()
+            nil, nil, w, h = self.get_geometry()
+            self.__tile_background(w, h)
+            
+        else:
+            self.__set_color("#ffffffa0")
 
 
 
@@ -185,26 +200,28 @@ class TargetGroup(ContainerTarget):
 
 
 
-    def get_target_at(self, px, py, is_root = 0):
+    def get_target_at(self, px, py, path, is_root = 0):
 
         x, y, w, h = self.get_geometry()
+        x, y = self.get_anchored_coords(x, y, w, h)
 
         if (is_root): x, y = 0, 0
         if (x <= px <= x + w and y <= py <= y + h):
             targets = self.__zones.get(px - x, py - y)
-            ctargets, cpath = [], []
+            ctargets = []
             
             children = self._get_children()
             children.reverse()
             for t in children:
                 if (t in targets):
-                    ctargets, cpath = t.get_target_at(px - x , py - y)
-                    if (ctargets): break
+                    ctargets += t.get_target_at(px - x , py - y, path)
+                    #if (ctargets): break
             #end for
 
-            return ([self] + ctargets, cpath)
+            ret = [(self, path)] + ctargets
+            return ret
 
         else:
-            return ([], [])
+            return []
         
         #end if
