@@ -2,13 +2,18 @@ from utils.Observable import Observable
 from utils.TargetSettings import TargetSettings
 from utils.TypeConverter import TypeConverter
 from utils import i18n
+from utils import singleton
+from config.ConfigManager import ConfigManager
 from SensorConfigurator import SensorConfigurator
 from Menu import Menu
 from main import _
+from main import PURGE_KEY
+from main import admin
 
 import gtk
 import thread
 import Queue
+import os
 
 
 
@@ -21,17 +26,33 @@ class Sensor(Observable):
     OBS_CMD_CONFIGURE = 1
     OBS_CMD_REMOVE = 2
     OBS_CMD_RESTART = 3
+    OBS_CMD_DUPLICATE = 4
+
+
+    def __del__(self):
+
+        print "DEL", self
+
 
 
     def __init__(self):
 
+        # the path of this sensor
+        self.__path = os.getcwd()
+
         # configuration manager for sensor properties
-        self.__config_manager = None
+        self.__config_manager = singleton.get(ConfigManager)
         self.__config_id = ""
 
+        # the ID of this sensor
+        self.__sensor_id = ""
+
+        # the configuration watcher callback
+        self.__config_watcher = None
+
         # flag for stopping the sensor
         self.__stop_flag = 0
-        
+
 
         # converter for data types
         self.__type_converter = TypeConverter()
@@ -39,6 +60,43 @@ class Sensor(Observable):
 
 
 
+    #
+    # Reacts on changes in the configuration and delegates the call to the
+    # user's callback.
+    #
+    def __on_watch_config(self, path, value):
+        assert(self.__config_watcher)
+
+        key = path
+        apply(self.__config_watcher, [key, value])
+
+
+
+    #
+    # Sets the ID of this sensor.
+    #
+    def set_id(self, id):
+
+        self.__id = id
+
+
+    #
+    # Returns the ID of this sensor. The ID is not yet valid in the
+    # constructor.
+    #
+    def _get_id(self):
+        assert(self.__id, "The ID is invalid in the constructor.")
+
+        return self.__id
+
+
+    #
+    # Returns the path of this sensor.
+    #
+    def _get_path(self): return self.__path
+
+
+
     #
     # Returns a new empty TargetSettings object.
     #
@@ -82,9 +140,12 @@ class Sensor(Observable):
             thread.start_new_thread(threadfunction, args)
 
         gtk.idle_add(run_thread, threadfunction, args)
-        
 
 
+
+    #
+    # Sends an action to this sensor.
+    #
     def send_action(self, call, path, args = []):
 
         if (not self.__stop_flag):
@@ -92,19 +153,26 @@ class Sensor(Observable):
 
 
 
+    #
+    # Method for handling action calls from the display. Sensors have to
+    # override this method.
+    #
     def call_action(self, call, path, args = []): raise NotImplementedError
 
 
 
     #
-    # Stops this sensor.
+    # Stops this sensor and cleans up.
     #
-    def stop(self, restart = 0):
+    def stop(self):
 
         self.__stop_flag = 1
+        del self.__config_watcher
+        self.__config_manager.remove_watcher(self._get_config_id(), "",
+                                             self.__on_watch_config)
 
-        # clear GConf
-        if (not restart): self.__config_manager.clear(self.__config_id)
+        # shut down sensor
+        self._shutdown()
 
 
 
@@ -116,10 +184,43 @@ class Sensor(Observable):
 
 
 
-    def set_config_manager(self, cm, config_id):
+    #
+    # Executes tasks for sensor shutdown. Sensors may override this method.
+    #
+    def _shutdown(self): pass
+
+
+
+    #
+    # Sets the configuration id for this sensor. If the user has set a callback
+    # function for watching the config, the config will be watched.
+    #
+    def set_config_id(self, config_id):
 
         self.__config_id = config_id
-        self.__config_manager = cm
+        if (self.__config_watcher):
+            self.__config_manager.watch(self.__config_id, "",
+                                        self.__on_watch_config)
+
+
+
+
+    #
+    # Returns the unique config ID of this sensor.
+    #
+    def _get_config_id(self): return self.__config_id
+
+
+
+    #
+    # Registers a watcher for config changes.
+    #
+    def _watch_config(self, callback):
+
+        self.__config_watcher = callback
+        if (self._get_config_id()):
+            self.__config_manager.watch(self._get_config_id(), "",
+                                        self.__on_watch_config)
 
 
 
@@ -137,9 +238,9 @@ class Sensor(Observable):
 
         value = self.__config_manager.get(self.__config_id, key)
         if (value == ""): value = self.__defaults.get(key, "")
-        
+
         return self.__type_converter.str2type(key, value)
-    
+
 
 
     #
@@ -151,6 +252,15 @@ class Sensor(Observable):
         self.__defaults[key] = default
 
 
+    #
+    # Sets the paths to remove when the config gets purged.
+    #
+    def _set_paths_to_purge(self, paths):
+
+        key = PURGE_KEY
+        value = ",".join(paths)
+        self.__config_manager.set(self.__config_id, key, value)
+
 
 
     #
@@ -167,6 +277,15 @@ class Sensor(Observable):
 
 
 
+    #
+    # Method for duplicating the display using this sensor
+    #
+    def _duplicate_display(self):
+
+        self.update_observer(self.OBS_CMD_DUPLICATE)
+
+
+
     #
     # Opens the given menu.
     #
@@ -208,7 +327,7 @@ class Sensor(Observable):
 
         from main import NAME, VERSION, COPYRIGHT, DESCRIPTION
         from main import AUTHORS, DOCUMENTERS, ICON
-        
+
         import gnome.ui
 
         tcredits = _("translator_credits")
